Attribute VB_Name = "bErrorHandler"
Option Explicit

Public Type tApplicationDefinedError
  ErrNumber As Long
  ErrMessage As String
  DialogType As Long
  LogError As Boolean
End Type
Public ApplicationDefinedError As tApplicationDefinedError

' Terminal error is generally used for database apps that encounter
' an error that is potentially dangerous to the database.
' To prevent corruption of the database, the app is closed when
' encountering one of these types of errors.
' Raising Terminal errors should be avoided.

Public TERMINAL_ERROR As Boolean

Public Const ERROR_NUMBER_BASE As Long = 40000

Public Const ERR_CANCEL_WAS_SELECTED As Long = 32755 ' Error returned when user selects cancel in a common dialog
Public Const ERR_BAD_FILENAME_OR_NUMBER As Long = 52
Public Const ERR_FILE_NOT_FOUND As Long = 53
Public Const ERR_BAD_FILE_MODE As Long = 54
Public Const ERR_FILE_ALREADY_OPEN As Long = 55
Public Const ERR_DEVICE_IO As Long = 57
Public Const ERR_DISK_FULL As Long = 61
Public Const ERR_INPUT_PAST_END_OF_FILE As Long = 62
Public Const ERR_DEVICE_UNAVAILABLE As Long = 68
Public Const ERR_PERMISSION_DENIED As Long = 70
Public Const ERR_DISK_NOT_READY As Long = 71
Public Const ERR_PATH_FILE_ACCESS As Long = 75
Public Const ERR_PATH_DOES_NOT_EXIST As Long = 76
Public Const ERR_PRINTER_ERROR As Long = 482

Public Function ErrorHandler(sError As String, nErrVal As Long, FileOrObject As Variant, sProcedure As String) As Long

On Error Resume Next

With ApplicationDefinedError
  .ErrNumber = nErrVal
  .DialogType = vbExclamation
End With

With ApplicationDefinedError
  .LogError = True ' Most of these errors are system errors so log error as default
  Select Case nErrVal
    Case ERR_BAD_FILE_MODE
      .ErrMessage = "Can't open file " & DBL_QUOTE & FileOrObject & DBL_QUOTE & " for access."
     .DialogType = vbRetryCancel
    Case ERR_FILE_ALREADY_OPEN
      .ErrMessage = "This file is already open."
      .DialogType = vbRetryCancel
    Case ERR_DEVICE_IO
      .ErrMessage = "Internal Disk Error."
     .DialogType = vbRetryCancel
    Case ERR_DISK_FULL
      .ErrMessage = "Disk is full.  Free some disk space and try again."
      .DialogType = vbRetryCancel
    Case ERR_INPUT_PAST_END_OF_FILE
      .ErrMessage = "This file has a non-standard end-of-file marker, or an attempt was made to read beyond the end-of-file marker."
      .DialogType = vbRetryCancel
    Case ERR_BAD_FILENAME_OR_NUMBER
      .ErrMessage = "Bad file name or number."
      .DialogType = vbRetryCancel + vbCritical
    Case ERR_FILE_NOT_FOUND
      .ErrMessage = "File " & DBL_QUOTE & FileOrObject & DBL_QUOTE & " not found."
      .DialogType = vbRetryCancel
    Case ERR_DEVICE_UNAVAILABLE
      .ErrMessage = "Device is currently unavailable."
      .DialogType = vbRetryCancel
    Case ERR_DISK_NOT_READY
      .ErrMessage = "There is no disk in the drive." & vbCrLf
      .ErrMessage = .ErrMessage & "Please insert a disk in the drive and try again."
      .DialogType = vbRetryCancel
    Case ERR_PATH_FILE_ACCESS
      .ErrMessage = "Could not access path. The drive or path specified is not available for the requested operation."
      .DialogType = vbRetryCancel
    Case ERR_PATH_DOES_NOT_EXIST
      .ErrMessage = "The specified path does not exist."
      .DialogType = vbRetryCancel
    Case ERR_PRINTER_ERROR
      .ErrMessage = "An error has occurred with your printer."
      .DialogType = vbRetryCancel
    Case ERR_PERMISSION_DENIED
      .ErrMessage = sError & vbCrLf & "Try again?"
      .DialogType = vbRetryCancel
    Case Is >= 40000
      ShowApplicationError nErrVal, FileOrObject
    Case Else
      .ErrMessage = sError
      .DialogType = vbInformation
      .LogError = True
  End Select

  If .ErrMessage = vbNullString Then .ErrMessage = sError
  ErrorHandler = MsgBox(.ErrMessage, .DialogType + vbMsgBoxHelpButton, APP_TITLE & ": Error " & nErrVal)

  If .LogError Then
    LogError sError, .ErrNumber, FileOrObject, sProcedure
  End If
End With

Err.Clear

If TERMINAL_ERROR Then
  MsgBox "An event occured that requires " & APP_TITLE & " to be shut down.  If this problem persists, please see your Administrator.", vbCritical, APP_TITLE
  ' Shut down the application
  EndApplication
End If

End Function
Private Sub CheckLogFileSize()
Dim nReturn As Long
Dim sMsg As String
Dim nFileLen As Long

On Error GoTo errHandler

If Not FileExists(APP_LOG) Then Exit Sub

nFileLen = FileLen(APP_LOG)
If nFileLen < 100000 Then Exit Sub

sMsg = "The " & APP_TITLE & " log file:  " & APP_LOG & DBL_RETURN & "has reached " & Format$(nFileLen / 1000, "#0") & " Kb in size.  If " & APP_TITLE & " does not seem to be working properly you should attach this file to an e-mail and send it to the Author (See E-mail Author in the Help menu)."
sMsg = sMsg & DBL_RETURN & APP_TITLE & " can delete this file now to free space on your hard drive.  A new log file will be generated next time you start the application."
sMsg = sMsg & DBL_RETURN & "If you would like to view the contents of this file, click " & DBL_QUOTE & "No" & DBL_QUOTE & " to cancel this dialog and select View Log File from the View Menu next time you log on."
sMsg = sMsg & DBL_RETURN & "Would you like to delete the file now?"

nReturn = MsgBox(sMsg, vbYesNo + vbQuestion, APP_TITLE)
If nReturn = vbNo Then Exit Sub

ShellDelete APP_LOG

Exit Sub

errHandler:
MsgBox Error

End Sub
Public Sub CloseLogFile()
Dim iFilenum As Integer

On Error GoTo errHandler

iFilenum = FreeFile
Open APP_LOG For Append Lock Read Write As #iFilenum
  Print #iFilenum, "Session ended:   " & Format(Now(), "Long Date") & " @" & Format(Now(), "Long Time")
  Print #iFilenum, String(60, "_")
Close #iFilenum

Exit Sub

errHandler:
LogError Error, Err, APP_LOG, "bErrorHandler.CloseLogFile()"

End Sub
Public Sub LogError(sError As String, nErrVal As Long, FileOrObject As Variant, sProcedure As String)
Dim iFilenum As Integer
Dim sErrorString As String

' Writes the error to a log file

On Error GoTo errHandler

If Not FileExists(APP_LOG) Then LogIntroduction

sErrorString = "     " & sError & ", error number: " & Trim$(Str$(nErrVal)) & " occurred in " & sProcedure & " on " & Format(Now(), "General date") & ".  " & APP_TITLE & " " & APP_VERSION
iFilenum = FreeFile
Open APP_LOG For Append Lock Read Write As #iFilenum
  Print #iFilenum, sErrorString
  Print #iFilenum, "Associated file or object: " & FileOrObject
Close #iFilenum

Exit Sub

errHandler:
Close

End Sub
Private Sub LogIntroduction()
Dim iFilenum As Integer
Dim sMsg As String

On Error GoTo errHandler

sMsg = APP_LOG & " logs system and critical application errors.  Entries are logged chronologically from the beginning to the end of the file.  " _
  & "This file will grow very large over time and should be purged periodically.  " _
  & "If you are having problems using " & APP_TITLE & " please attach a copy of this file " _
  & "to an e-mail and send it to the developer, " & SUPPORT_EMAIL & CHAR_PERIOD

sMsg = sMsg & DBL_RETURN & String(60, "_") & vbCrLf

iFilenum = FreeFile
Open APP_LOG For Output As #iFilenum
  Print #iFilenum, sMsg
Close #iFilenum

Exit Sub

errHandler:
Close

End Sub
Public Sub OpenLogFile()
Dim iFilenum As Integer

On Error GoTo errHandler

CheckLogFileSize

If Not FileExists(APP_LOG) Then LogIntroduction

iFilenum = FreeFile

Open APP_LOG For Append Lock Read Write As #iFilenum
  Print #iFilenum, "Session started:   " & Format(SessionStartTime, "Long Date") & " @" & Format(SessionStartTime, "Long Time")
  Print #iFilenum, vbNullString
Close #iFilenum

Exit Sub

errHandler:
LogError Error, Err, APP_LOG, "bErrorHandler.OpenLogFile()"

End Sub
Public Sub ViewLogFile(hwnd As Long)

On Error GoTo errHandler

ShellExecute hwnd, "open", APP_LOG, vbNullString, vbNullString, 3

Exit Sub

errHandler:
MsgBox Error

End Sub
